﻿/*
One of those Weeks Episode 37
Copyright (c) 2008,2011 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.ep37
{
	// Adobe Imports
	import flash.display.MovieClip;
	import flash.display.SimpleButton;
	import flash.events.*;

	import com.blazinggames.ootw.inventory.*;

	public class CircuitGame
	extends MovieClip
	{
// ***** CONSTANTS **********************************************************

		public static const CB_STRING:String = ".r7LJ-|T^+*ABCDE";
		public static const LEVEL_COMPLETE:String = "LevelCompleteEvent";

		private static const OFFSET_X:int = 10;
		private static const OFFSET_Y:int = 15;
// ***** VARIAIBLES *********************************************************

		public var solution:Array = new Array(25);
		public var circuits:Array = new Array(25);
		public var hotspots:Array = new Array(25);
		
		public var parts:Array = new Array(11);
		
		// from flash: bar_movie, con_btn, status_movie

// ***** CONSTRUCTOR ********************************************************

		public function CircuitGame()
		{
			var cntr:int;
			
			bar_movie.addEventListener(InventoryConst.INVENTORY_EVENT, handleInventory);
			for (cntr = 0; cntr < 25; ++cntr) {
				circuits[cntr] = new CircuitPiece();
				circuits[cntr].x = 75 * Math.floor(cntr % 5) + OFFSET_X;
				circuits[cntr].y = 75 * Math.floor(cntr / 5) + OFFSET_Y;
				hotspots[cntr] = new CircuitHotspot();
				hotspots[cntr].x = circuits[cntr].x;
				hotspots[cntr].y = circuits[cntr].y;
				hotspots[cntr].addEventListener(MouseEvent.CLICK,handleHotspot);
				addChild(circuits[cntr]);
				addChild(hotspots[cntr]);
			}
			for (cntr = 0; cntr < 11; ++cntr) {
				var partIcon:CircuitPiece = new CircuitPiece();
				partIcon.width = 40;
				partIcon.height = 40;
				partIcon.setTile(cntr);
				parts[cntr] = new BasicInventoryItem(null, partIcon, "Part "+ cntr);
			}
			con_btn.addEventListener(MouseEvent.CLICK, handleContinue);
		}

// ***** Functions - Event handlers *****************************************

		public function handleInventory(evt:Event)
		{
			trace("Inventory event:");
			trace("   State is now: " + bar_movie.getInventoryState());
			var itm:IInventoryItem = bar_movie.getSelectedItem();
			if (itm != null)
				trace("   Selected item is " + itm.getItemName());
			itm = bar_movie.getSecondaryItem();
			if (itm != null){
				trace("   Combined item is " + itm.getItemName());
				bar_movie.setInventoryState(InventoryConst.INVSTATE_SELECTED, itm);
			}
		}
		
		public function handleHotspot(evt:MouseEvent)
		{
			var cntr:int, spot:int = -1;
			for (cntr = 0; cntr < 25; ++cntr)
				if (evt.target == hotspots[cntr])
					spot = cntr;
			if (spot >= 0) {
				trace("Circuit " + spot + " clicked on.");
				var itm:IInventoryItem = bar_movie.getSelectedItem();
				var pieceType = 0;
				for (cntr = 0; cntr < parts.length; ++cntr)
					if (itm == parts[cntr])
						pieceType = cntr;
				circuits[spot].setTile(pieceType);
			}
			updateGameState();
		}
		
		public function handleContinue(evt:MouseEvent)
		{
			var evnt:Event = new Event(LEVEL_COMPLETE);
			this.dispatchEvent(evnt);
		}

// ***** Functions - Getters and Setters ************************************

		public function updateGameState()
		{
			var tileID:int, cntr:int;
			//var newState = CircuitStatus.MISSING_PARTS;
			var missingParts:Boolean = false;
			var wrongParts:Boolean = false;
			
			for (cntr = 0; cntr < 25; ++cntr) {
				tileID = circuits[cntr].getTile();
				if (tileID != solution[cntr]) {
					if (tileID == CircuitPiece.EMPTY)
						missingParts = true;
					else
						wrongParts = true;
				}
			}
			
			if (missingParts)
				status_movie.setState(CircuitStatus.MISSING_PARTS);
			else if (wrongParts)
				status_movie.setState(CircuitStatus.INVALID_CONFIG);
			else {
				con_btn.visible = true;
				for (cntr = 0; cntr < 25; ++cntr)
					hotspots[cntr].visible = false;
				status_movie.setState(CircuitStatus.CORRECT);
			}
		}
		public function setCircuit(layout:String, removed:Array, invList:Array)
		{
			var tileID:int, cntr:int;
			
			// set the circuit board/solution
			for (cntr = 0; cntr < 25; ++cntr) {
				tileID = CB_STRING.indexOf(layout.substring(cntr,cntr+1));
				circuits[cntr].setTile(tileID);
				solution[cntr] = tileID;
				hotspots[cntr].visible = false;
			}
			status_movie.setState(CircuitStatus.MISSING_PARTS);
			con_btn.visible = false;
			
			// clear out the removed items
			if (removed != null) {
				for (cntr = 0; cntr < removed.length; ++cntr) {
					circuits[removed[cntr]].setTile(0);
					hotspots[removed[cntr]].visible = true;
				}
			}
			
			// and set up the inventory
			bar_movie.clearInventory();
			if (invList != null) {
				for (cntr = 0; cntr < invList.length; ++cntr)
					bar_movie.addInvItem(parts[invList[cntr]]);
			}
		}
	} // end class
} // end package

